use microrm::{prelude::*, schema::entity::EntityID};
use crate::{schema, UIDCError};

#[derive(Debug)]
pub struct RoleInterface;

#[derive(Debug, clap::Subcommand)]
pub enum RoleCommands {
    Create { name: String }
}

impl microrm::cli::EntityInterface for RoleInterface {
    type Error = UIDCError;
    type Entity = schema::Role;
    type Context = microrm::schema::Stored<schema::Realm>;
    type CustomCommand = RoleCommands;

    fn run_custom(
        ctx: &Self::Context,
        cmd: Self::CustomCommand,
        query_ctx: impl Queryable<EntityOutput = Self::Entity> + Insertable<Self::Entity>,
    ) -> Result<(), Self::Error> {
        match cmd {
            RoleCommands::Create { name } => {
                query_ctx.insert(schema::Role {
                    realm: ctx.id(),
                    shortname: name,
                    groups: Default::default(),
                })?;
            },
        }

        Ok(())
    }

    fn should_override(_entity: &'static str, field: &'static str, _role: microrm::cli::ValueRole) -> bool {
        if field == "realm" {
            true
        }
        else {
            false
        }
    }

    fn override_for(
        ctx: &Self::Context,
        _entity: &'static str,
        field: &'static str,
        _role: microrm::cli::ValueRole,
    ) -> String {
        if field == "realm" {
            format!("{}", ctx.id().into_raw())
        }
        else {
            unreachable!()
        }
    }
}
