use microrm::{prelude::*, schema::entity::EntityID};
use crate::{schema, user_management, UIDCError};

#[derive(Debug)]
pub struct UserInterface;

#[derive(Debug, clap::Subcommand)]
pub enum UserCommands {
    Create { username: String },
    UpdateAuth {
        username: String,
        #[clap(short = 'p', long, action = clap::ArgAction::Count)]
        password: u8,
        #[clap(short = 't', long, action = clap::ArgAction::Count)]
        totp: u8,
    },
}

impl microrm::cli::EntityInterface for UserInterface {
    type Error = UIDCError;
    type Entity = schema::User;
    type Context = microrm::schema::Stored<schema::Realm>;
    type CustomCommand = UserCommands;

    fn run_custom(
        ctx: &Self::Context,
        cmd: Self::CustomCommand,
        query_ctx: impl Queryable<EntityOutput = Self::Entity> + Insertable<Self::Entity>,
    ) -> Result<(), Self::Error> {
        match cmd {
            UserCommands::Create { username } => {
                query_ctx.insert(schema::User {
                    realm: ctx.id(),
                    username,
                    auth: Default::default(),
                    groups: Default::default(),
                })?;
            },
            UserCommands::UpdateAuth { username, password, totp } => {
                user_management::change_auth(ctx.as_ref(), &username, password > 0, totp > 0)?;
            },
        }

        Ok(())
    }

    fn should_override(_entity: &'static str, field: &'static str, _role: microrm::cli::ValueRole) -> bool {
        if field == "realm" {
            true
        }
        else {
            false
        }
    }

    fn override_for(
        ctx: &Self::Context,
        _entity: &'static str,
        field: &'static str,
        _role: microrm::cli::ValueRole,
    ) -> String {
        if field == "realm" {
            format!("{}", ctx.id().into_raw())
        }
        else {
            unreachable!()
        }
    }
}
