use crate::{key::KeyType, schema, UIDCError};
use microrm::prelude::*;

pub fn create(realm: &microrm::Stored<schema::Realm>, name: &String, key_type: KeyType) -> Result<(), UIDCError> {
    let rng = ring::rand::SystemRandom::new();
    let client_secret: [u8; 32] = ring::rand::generate(&rng).unwrap().expose();

    realm.clients.insert(schema::Client {
        realm: realm.id(),
        shortname: name.into(),
        secret: base64::encode(&client_secret),
        key_type: key_type.into(),
        redirects: Default::default(),
        scopes: Default::default(),
    })?;
    Ok(())
}

pub fn rotate_secret(realm: &microrm::Stored<schema::Realm>, name: &str) -> Result<(), UIDCError> {
    let rng = ring::rand::SystemRandom::new();
    let client_secret: [u8; 32] = ring::rand::generate(&rng).unwrap().expose();

    todo!();

    Ok(())
}

pub fn inspect(realm: &microrm::Stored<schema::Realm>, name: &str) -> Result<(), UIDCError> {
    if let Some(client) = realm.clients.with(schema::Client::Shortname, name).first().get()? {
        println!("Found client {name}");
        println!("Client secret: {}", client.secret);
        println!("Signature type: {:?}", client.key_type);
        println!("Valid redirect URIs:");
        for redirect in client.redirects.get()? {
            println!(" - {}", redirect.redirect);
        }

        println!("Valid scopes:");
        for scope in client.scopes.get()? {
            println!(" - {}", scope.shortname);
        }
    } else {
        println!("No such client {name}");
    }

    Ok(())
}
